#!/bin/perl
#
$line="";
while (<STDIN>) {
	chomp;
	s/^\s+//;
	s/\s+/ /;
	if (/^%/) {
		print $line."\n";
		$line="";
	} else {
		if ($line ne "") {
			$line="$line $_";
		} else {
			$line=$_;
		}
	}
}
exit 0;
